@@ -35,10 +35,15 @@ from subprocess import call
 VERBOSE = False
 STATE_START, STATE_TEXT, STATE_WORDS, STATE_TREE, STATE_DEPENDENCY, STATE_COREFERENCE = 0, 1, 2, 3, 4, 5
 WORD_PATTERN = re.compile('\[([^\]]+)\]')
-CR_PATTERN = re.compile(r"\((\d*),(\d)*,\[(\d*),(\d*)\)\) -> \((\d*),(\d)*,\[(\d*),(\d*)\)\), that is: \"(.*)\" -> \"(.*)\"")
+#CR_PATTERN = re.compile(r"\((\d*),(\d)*,\[(\d*),(\d*)\]\) -> \((\d*),(\d)*,\[(\d*),(\d*)\]\), that is: \"(.*)\" -> \"(.*)\"")
 
-DIRECTORY = "stanford-corenlp-full-2013-06-20"
+# Fixed by Diego Reforgiato. Otherwise src_pos gets one digit only
+CR_PATTERN = re.compile(r"\((\d*),(\d*),\[(\d*),(\d*)\]\) -> \((\d*),(\d*),\[(\d*),(\d*)\]\), that is: \"(.*)\" -> \"(.*)\"")
 
+if os.environ.has_key("CORENLP"):
+    DIRECTORY = os.environ["CORENLP"]
+else:
+    DIRECTORY = "stanford-corenlp-full-2013-11-12"
 
 class bc:
     HEADER = '\033[95m'
@@ -91,15 +96,16 @@ def init_corenlp_command(corenlp_path, memory, properties):
     Spawns the server as a process.
     """
 
-    # TODO: Can edit jar constants
-    jars = ["stanford-corenlp-3.2.0.jar",
-            "stanford-corenlp-3.2.0-models.jar",
+    jars = ["stanford-corenlp-3.3.0.jar",
+            "stanford-corenlp-3.3.0-models.jar",
             "xom.jar",
             "joda-time.jar",
-            "jollyday.jar"]
+            "jollyday.jar",
+            "ejml-0.23.jar"] # No idea what this is but it might be sentiment
 
     java_path = "java"
     classname = "edu.stanford.nlp.pipeline.StanfordCoreNLP"
+
     # include the properties file, so you can change defaults
     # but any changes in output format will break parse_parser_results()
     current_dir_pr = os.path.dirname(os.path.abspath(__file__)) + "/" + properties
@@ -160,11 +166,15 @@ def parse_parser_results(text):
     """
     results = {"sentences": []}
     state = STATE_START
-    for line in unidecode(text.decode('utf-8')).split("\n"):
+
+    if type(text) == str:
+        text = text.decode('utf-8')
+
+    for line in unidecode(text).split("\n"):
         line = line.strip()
 
         if line.startswith("Sentence #"):
-            sentence = {'words': [], 'parsetree': [], 'dependencies': []}
+            sentence = {'words': [], 'parsetree': [], 'dependencies': [], 'indexeddependencies': []}
             results["sentences"].append(sentence)
             state = STATE_TEXT
 
@@ -194,6 +204,7 @@ def parse_parser_results(text):
                 if len(split_entry) == 3:
                     rel, left, right = map(lambda x: remove_id(x), split_entry)
                     sentence['dependencies'].append(tuple([rel, left, right]))
+                    sentence['indexeddependencies'].append(tuple(split_entry))
 
         elif state == STATE_COREFERENCE:
             if "Coreference set" in line:
@@ -214,9 +225,20 @@ def parse_parser_xml_results(xml, file_name="", raw_output=False):
     import xmltodict
     from collections import OrderedDict
 
+    def enforceList(list_or_ordered_dict): #TIM
+        if type(list_or_ordered_dict) == type(OrderedDict()):
+            return [list_or_ordered_dict]
+        else:
+            return list_or_ordered_dict
+
     def extract_words_from_xml(sent_node):
-        exted = map(lambda x: x['word'], sent_node['tokens']['token'])
-        return exted
+        if type(sent_node['tokens']['token']) == type(OrderedDict()):
+            # This is also specific case of xmltodict
+            exted = [sent_node['tokens']['token']]
+        else:
+            exted = sent_node['tokens']['token']
+        exted_string = map(lambda x: x['word'], exted)
+        return exted_string
 
     # Turning the raw xml into a raw python dictionary:
     raw_dict = xmltodict.parse(xml)
@@ -235,11 +257,20 @@ def parse_parser_xml_results(xml, file_name="", raw_output=False):
         # Making a raw coref dictionary:
         raw_coref_list = document[u'coreference'][u'coreference']
 
+        # It is specific case that there is only one item for xmltodict
+        if len(raw_coref_list) == 1:
+            raw_coref_list = [raw_coref_list]
+        if len(raw_sent_list) == 1:
+            raw_sent_list = [raw_sent_list]
+
+        # This is also specific case of xmltodict
+        raw_sent_list = enforceList(raw_sent_list)
+
         # To dicrease is for given index different from list index
-        coref_index = [[[int(raw_coref_list[j][u'mention'][i]['sentence']) - 1,
-                         int(raw_coref_list[j][u'mention'][i]['head']) - 1,
-                         int(raw_coref_list[j][u'mention'][i]['start']) - 1,
-                         int(raw_coref_list[j][u'mention'][i]['end']) - 1]
+        coref_index = [[[int(raw_coref_list[j]['mention'][i]['sentence']) - 1,
+                         int(raw_coref_list[j]['mention'][i]['head']) - 1,
+                         int(raw_coref_list[j]['mention'][i]['start']) - 1,
+                         int(raw_coref_list[j]['mention'][i]['end']) - 1]
                         for i in xrange(len(raw_coref_list[j][u'mention']))]
                        for j in xrange(len(raw_coref_list))]
 
@@ -247,7 +278,11 @@ def parse_parser_xml_results(xml, file_name="", raw_output=False):
         for j in xrange(len(coref_index)):
             coref_list.append(coref_index[j])
             for k, coref in enumerate(coref_index[j]):
-                exted = raw_sent_list[coref[0]]['tokens']['token'][coref[2]:coref[3]]
+                if type(raw_sent_list[coref[0]]['tokens']['token']) == type(OrderedDict()):
+                    # This is also specific case of xmltodict
+                    exted = [raw_sent_list[coref[0]]['tokens']['token']]
+                else:
+                    exted = raw_sent_list[coref[0]]['tokens']['token'][coref[2]:coref[3]]
                 exted_words = map(lambda x: x['word'], exted)
                 coref_list[j][k].insert(0, ' '.join(exted_words))
 
@@ -257,26 +292,48 @@ def parse_parser_xml_results(xml, file_name="", raw_output=False):
     else:
         coref_flag = False
 
-    # Convert sentences to the format like python
-    # TODO: If there is only one sentence in input sentence,
-    # raw_sent_list is dict and cannot decode following code...
-    sentences = [{'dependencies': [[dep['dep'][i]['@type'],
-                                    dep['dep'][i]['governor']['#text'],
-                                    dep['dep'][i]['dependent']['#text']]
-                                   for dep in raw_sent_list[j][u'dependencies']
-                                   for i in xrange(len(dep['dep']))
-                                   if dep['@type'] == 'basic-dependencies'],
-                  'text': extract_words_from_xml(raw_sent_list[j]),
-                  'parsetree': str(raw_sent_list[j]['parse']),
-                  'words': [[str(token['word']), OrderedDict([
-                      ('NamedEntityTag', str(token['NER'])),
-                      ('CharacterOffsetEnd', str(token['CharacterOffsetEnd'])),
-                      ('CharacterOffsetBegin', str(token['CharacterOffsetBegin'])),
-                      ('PartOfSpeech', str(token['POS'])),
-                      ('Lemma', str(token['lemma']))])]
-                  for token in raw_sent_list[j][u'tokens'][u'token']]}
+    # This is also specific case of xmltodict
+    raw_sent_list = enforceList(raw_sent_list)
 
-                 for j in xrange(len(raw_sent_list))]
+    sentences = []
+    for id in xrange(len(raw_sent_list)):
+        sent = {}
+        sent['text'] = extract_words_from_xml(raw_sent_list[id])
+        sent['parsetree'] = unicode(raw_sent_list[id]['parse'])
+        # sent['sentimentValue'] = int(raw_sent_list[id].get(['@sentimentValue'])) # TIM
+        # sent['sentiment'] = raw_sent_list[id]['@sentiment'] # TIM
+        sentiment_value = raw_sent_list[id].get('@sentimentValue')
+        sentiment = raw_sent_list[id].get('@sentiment')
+        if sentiment_value: sent['sentimentValue'] = int(sentiment_value)
+        if sentiment_value: sent['sentiment'] = sentiment
+
+        if type(raw_sent_list[id]['tokens']['token']) == type(OrderedDict()):
+            # This is also specific case of xmltodict
+            token = raw_sent_list[id]['tokens']['token']
+            sent['words'] = [
+                [unicode(token['word']), OrderedDict([
+                    ('NamedEntityTag', str(token['NER'])),
+                    ('CharacterOffsetEnd', str(token['CharacterOffsetEnd'])),
+                    ('CharacterOffsetBegin', str(token['CharacterOffsetBegin'])),
+                    ('PartOfSpeech', str(token['POS'])),
+                    ('Lemma', unicode(token['lemma']))])]
+            ]
+        else:
+            sent['words'] = [[unicode(token['word']), OrderedDict([
+                ('NamedEntityTag', str(token['NER'])),
+                ('CharacterOffsetEnd', str(token['CharacterOffsetEnd'])),
+                ('CharacterOffsetBegin', str(token['CharacterOffsetBegin'])),
+                ('PartOfSpeech', str(token['POS'])),
+                ('Lemma', unicode(token['lemma']))])]
+                             for token in raw_sent_list[id]['tokens']['token']]
+
+        sent['dependencies'] = [[enforceList(dep['dep'])[i]['@type'],
+                                 enforceList(dep['dep'])[i]['governor']['#text'],
+                                 enforceList(dep['dep'])[i]['dependent']['#text']]
+                                for dep in raw_sent_list[id]['dependencies'] if dep.get('dep')
+                                for i in xrange(len(enforceList(dep['dep'])))
+                                if dep['@type'] == 'basic-dependencies']
+        sentences.append(sent)
 
     if coref_flag:
         results = {'coref': coref_list, 'sentences': sentences}
@@ -342,7 +399,7 @@ class StanfordCoreNLP:
     def _spawn_corenlp(self):
         if VERBOSE:
             print self.start_corenlp
-        self.corenlp = pexpect.spawn(self.start_corenlp, maxread=8192, searchwindowsize=80)
+        self.corenlp = pexpect.spawn(self.start_corenlp, timeout=60, maxread=8192, searchwindowsize=80)
 
         # show progress bar while loading the models
         if VERBOSE:
